@@ -789,20 +789,25 @@ class _Pickler:
         if self.proto < 5:
             raise PicklingError("PickleBuffer can only pickled with "
                                 "protocol >= 5")
-        with memoryview(obj) as m:
+        with obj.raw() as m:
+            if not m.contiguous:
+                raise PicklingError("PickleBuffer can not be pickled when "
+                                    "pointing to a non-contiguous buffer")
+            in_band = True
             if self._buffer_callback is not None:
+                in_band = bool(self._buffer_callback(obj))
+            if in_band:
+                # Write data in-band
+                # XXX we could avoid a copy here
+                if m.readonly:
+                    self.save_bytes(m.tobytes())
+                else:
+                    self.save_bytearray(m.tobytes())
+            else:
                 # Write data out-of-band
-                self._buffer_callback([obj])
                 self.write(NEXT_BUFFER)
                 if m.readonly:
                     self.write(READONLY_BUFFER)
-            else:
-                # Write data in-band
-                # TODO avoid copy here
-                if m.readonly:
-                    self.save_bytes(bytes(m))
-                else:
-                    self.save_bytearray(bytearray(m))
 
     dispatch[PickleBuffer] = save_picklebuffer
 
